import { defineStore } from "pinia";
import {
  BucketPermissionService,
  BucketService,
  Constraint,
} from "@/client/s3proxy";
import type {
  BucketOut,
  BucketIn,
  BucketPermissionOut,
} from "@/client/s3proxy";
import { useAuthStore } from "@/stores/auth";

export const useBucketStore = defineStore({
  id: "buckets",
  state: () =>
    ({
      buckets: [],
      ownPermissions: {},
    } as {
      buckets: BucketOut[];
      ownPermissions: Record<string, BucketPermissionOut>;
    }),
  getters: {
    permissionFeatureAllowed(): (bucketName: string) => boolean {
      return (bucketName) => {
        // If a permission for the bucket exist, then false
        if (this.ownPermissions[bucketName] != null) {
          return false;
        }
        const bucket = this.buckets.find((b) => bucketName == b.name);
        // If the bucket doesn't exist, then false
        if (bucket == null) {
          return false;
        }
        // If there is a constraint on the bucket, then false otherwise true
        return bucket.owner_constraint == null;
      };
    },
    writableBuckets(): BucketOut[] {
      return this.buckets.filter((bucket) => {
        const permission = this.ownPermissions[bucket.name];
        if (permission != null) {
          return permission.permission !== "READ";
        }
        // If the user owns the bucket, check the bucket constraint
        return (
          bucket.owner_constraint == null ||
          bucket.owner_constraint == Constraint.WRITE
        );
      });
    },
    getBucketPermission(): (
      bucketName: string
    ) => BucketPermissionOut | undefined {
      return (bucketName) => this.ownPermissions[bucketName];
    },
    writableBucket(): (bucketName: string) => boolean {
      return (bucketName) => {
        // If this is a foreign bucket, check that the user has write permission
        const permission = this.ownPermissions[bucketName];
        if (permission != null) {
          return permission.permission !== "READ";
        }
        // If the user owns the bucket, check the bucket constraint
        const bucket = this.buckets.find((b) => bucketName == b.name);
        if (bucket != null) {
          return (
            bucket.owner_constraint == null ||
            bucket.owner_constraint == Constraint.WRITE
          );
        }
        return false;
      };
    },
    readableBucket(): (bucketName: string) => boolean {
      return (bucketName) => {
        // If this is a foreign bucket, check that the user has read permission
        const permission = this.ownPermissions[bucketName];
        if (permission != null) {
          return permission.permission !== "WRITE";
        }
        // If the user owns the bucket, check the bucket constraint
        const bucket = this.buckets.find((b) => bucketName == b.name);
        if (bucket != null) {
          return (
            bucket.owner_constraint == null ||
            bucket.owner_constraint == Constraint.READ
          );
        }
        return false;
      };
    },
  },
  actions: {
    _fetchOwnPermissions(
      onFulfilled:
        | ((buckets: BucketPermissionOut[]) => void)
        | null
        | undefined = null,
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      onRejected: ((reason: any) => void) | null | undefined = null,
      onFinally: (() => void) | null | undefined = null
    ) {
      const authStore = useAuthStore();
      if (authStore.user != null && !authStore.foreignUser) {
        BucketPermissionService.bucketPermissionListPermissionsPerUser(
          authStore.user.uid
        )
          .then((permissions) => {
            const new_permissions: Record<string, BucketPermissionOut> = {};
            for (const perm of permissions) {
              new_permissions[perm.bucket_name] = perm;
            }
            this.ownPermissions = new_permissions;
            onFulfilled?.(permissions);
          })
          .catch(onRejected)
          .finally(onFinally);
      }
    },
    deleteOwnPermission(bucketName: string) {
      this.buckets = this.buckets.filter(
        (bucket) => bucket.name !== bucketName
      );
      delete this.ownPermissions[bucketName];
    },
    fetchBuckets(
      onFulfilled: ((buckets: BucketOut[]) => void) | null | undefined = null,
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      onRejected: ((reason: any) => void) | null | undefined = null,
      onFinally: (() => void) | null | undefined = null
    ) {
      const authStore = useAuthStore();
      BucketService.bucketListBuckets(authStore.user?.uid)
        .then((buckets) => {
          this.buckets = buckets;
          onFulfilled?.(buckets);
        })
        .catch(onRejected)
        .finally(onFinally);
      this._fetchOwnPermissions();
    },
    fetchBucket(
      bucketName: string,
      onFulfilled: ((bucket: BucketOut) => void) | null | undefined = null,
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      onRejected: ((reason: any) => void) | null | undefined = null,
      onFinally: (() => void) | null | undefined = null
    ) {
      BucketService.bucketGetBucket(bucketName)
        .then((bucket) => {
          this.buckets[
            this.buckets.findIndex(
              (tempBucket) => tempBucket.name === bucket.name
            )
          ] = bucket;
          onFulfilled?.(bucket);
        })
        .catch(onRejected)
        .finally(onFinally);
    },
    deleteBucket(
      bucketName: string,
      onFulfilled: (() => void) | null | undefined = null,
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      onRejected: ((reason: any) => void) | null | undefined = null,
      onFinally: (() => void) | null | undefined = null
    ) {
      BucketService.bucketDeleteBucket(bucketName, true)
        .then(() => {
          this.buckets = this.buckets.filter(
            (bucket) => bucket.name !== bucketName
          );
          onFulfilled?.();
        })
        .catch(onRejected)
        .finally(onFinally);
    },
    createBucket(
      bucket: BucketIn,
      onFulfilled:
        | ((createdBucket: BucketOut) => void)
        | null
        | undefined = null,
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      onRejected: ((reason: any) => void) | null | undefined = null,
      onFinally: (() => void) | null | undefined = null
    ) {
      BucketService.bucketCreateBucket(bucket)
        .then((createdBucket) => {
          this.buckets.push(createdBucket);
          this.buckets.sort((bucketA, bucketB) =>
            bucketA.name >= bucketB.name ? 1 : -1
          );
          onFulfilled?.(createdBucket);
        })
        .catch(onRejected)
        .finally(onFinally);
    },
  },
});
